/**
 * Model for representing Finch sensors (specifically, light and temperature)
 * 
 * <P>
 * Services include:
 * <UL>
 * <LI> Supports adding and removing model listeners
 * <LI> The model will query the Finch for updates when told to do
 * so by the controller
 * <LI> On update, the model will generate ActionEvents
 * </UL>
 * <P>
 * 
 * @author Andrew H. Fagg  and you
 */

import finch.Finch;
import java.util.*;
import java.awt.event.*;

public class FinchModel {
	/**
	 * Reference to the Finch object
	 */
	private Finch myFinch;
	
	/**
	 * Local copy of the current temperature
	 */
	private double temperature;
	
	/**
	 * Local copy of the current light levels
	 */
	private int[] light = new int[2];
	
	/**
	 * List of ActionListeners
	 */
	private ArrayList<ActionListener> actionListenerList;
	
	// Random number generator: used to generate test values
	//   when we do not have a Finch connected
	private Random generator = new Random();
	
	/**
	 * Constructor
	 * 
	 * @param myFinch Reference to the Finch object.  If null, then assumes
	 * that no Finch is connected (and will fill in random data)
	 */
	public FinchModel(Finch myFinch){
		this.myFinch = myFinch;
	};
	
	
	/**
	 * Called by controller to force the model to update its currently
	 * stored light levels.
	 * 
	 * <P>
	 * TODO: complete implementation
	 */
	public void updateLight(){
		if(myFinch == null){
			// No Finch: randomly generate something
			light[0] = generator.nextInt() % 200;
			light[1] = generator.nextInt() % 200;
		}else{
			// Ask the Finch for the values
			light = myFinch.getLightSensors();
		}
	}
	
	/**
	 * Called by controller to force the model to update its currently
	 * stored temperature level
	 *
	 * <P>
	 * TODO: Complete implementation
	 */
	
	public void updateTemperature(){
		if(myFinch == null){
			// No Finch: randomly generate something
			temperature = generator.nextDouble();
		}else{
			// Query the Finch for the value
			temperature = myFinch.getTemperature();
		}
	}
	
	/**
	 * Accessor
	 * 
	 * @return Currently stored temperature value
	 */
	public double getTemperature(){
		return(temperature);
	}
	
	/**
	 * Accessor
	 * 
	 * @return Currently stored light values
	 */
	public int[] getLight(){
		return(light);
	}
	
	/**
	 * Add an ActionListener to the listener list
	 * 
	 * <P> 
	 * Note: this method is declared as "synchronized" so that only one 
	 * thread is operating on the actionListenerList at once (each
	 * of our Window objects have their own threads)
	 * 
	 * <P>
	 * 
	 * Implementation from Listing 32.2 of Introduction to Java (Liang)
	 * 
	 * @param listener Reference to the listener to be added
	 * 
	 */
	public synchronized void addActionListener(ActionListener listener){
		// Does the list exist?
		if(actionListenerList == null) {
			// No: create an empty one (with two open slots)
			actionListenerList = new ArrayList<ActionListener>(2);
		}
		
		// Does the current listener exist in the list already?
		if(!actionListenerList.contains(listener)){
			// No: add it
			actionListenerList.add(listener);
		}
	}
	
	/**
	 * Remove an ActionListener to the listener list
	 * 
	 * Implementation from Listing 32.2 of Introduction to Java (Liang)
	 * 
	 * @param listener Listener to be removed
	 */
	public synchronized void removeActionListener(ActionListener listener){
		//	 Does the list exist and is the listener contained in that list?
		if(actionListenerList != null && actionListenerList.contains(listener)){
			// Yes to both: remove this listener
			actionListenerList.remove(listener);
		}
	}
	
	/**
	 * Alert the listeners that a new event has occurred
	 * 
	 * Implementation from Listing 32.2 of Introduction to Java (Liang)
	 * 
	 * @param e ActionEvent to be sent
	 */
	private void processEvent(ActionEvent e){
		ArrayList list;
		
		// The following block must only have one thread
		// executing it at any one time.
		synchronized(this){
			// If the list does not exist, then we are done
			if(actionListenerList == null) return;
			// Create a local copy for this method call/thread
			list = (ArrayList)actionListenerList.clone();
		}
		
		// Loop over each listener and call the actionPerformed() method
		for(int i = 0; i < list.size(); ++i){
			ActionListener listener = (ActionListener) list.get(i);
			listener.actionPerformed(e);
			
		}
	}
}
